<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Models\Bill;
use App\Models\Expense;
use App\Models\PurchaseOrder;
use Carbon\Carbon;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class DashboardController extends Controller
{
    /**
     * Handle the incoming request.
     */
    public function __invoke(Request $request): JsonResponse {
        // Validate the request
        $request->validate([
            "start_date" => "required|date",
            "end_date" => "required|date",
        ]);

        $franchiseId = auth()->user()->franchise_id;

        // parse start and end dates
        $startDate = Carbon::parse($request->input('start_date'))->startOfDay();
        $endDate = Carbon::parse($request->input('end_date'))->endOfDay();

        $applyFilters = fn($query) => $query->where('franchise_id', $franchiseId)
            ->whereBetween('created_at', [$startDate, $endDate]);

        // fetch all bills at once
        $bills = $applyFilters(Bill::query())->get();

        // get counts and totals
        $billCount = $bills->count();
        $totalBillAmount = $bills->sum('amount');

        // bill count where table_id is not null
        $tableBillsCount = $bills->filter(fn($bill) => $bill->table_id !== null)->count();

        // total time spent
        $totalTimeSpent = $applyFilters(Bill::query())
            ->whereNotNull("table_id")
            ->selectRaw('SUM(TIMESTAMPDIFF(SECOND, created_at, completed_at)) as total_time')
            ->value('total_time');

        // Purchase orders and expenses
        $totalPurchaseOrdersCost = $applyFilters(PurchaseOrder::query())->sum('total');
        $totalExpensesCost = $applyFilters(Expense::query())->sum('amount');

        // Calculate averages
        $avgTimeSpent = $tableBillsCount > 0 ? ($totalTimeSpent / $tableBillsCount) / 60 : 0; // Convert to minutes
        $avgOrderValue = $billCount > 0 ? $totalBillAmount / $billCount : 0;

        // Return the data as JSON
        return response()->json([
            "totalSale" => number_format($totalBillAmount, 2),
            "totalExpense" => number_format($totalPurchaseOrdersCost + $totalExpensesCost, 2),
            "averageOrderValue" => number_format($avgOrderValue, 2),
            "totalBill" => number_format($totalBillAmount, 2),
            "avgTimeSpent" => number_format($avgTimeSpent, 2)
        ]);
    }
}
